@@ -2,8 +2,10 @@
 #define CODE_BAMbinSortUnmapped
 #include "IncludeDefine.h"
 #include "Parameters.h"
+#include "Genome.h"
+
 #include SAMTOOLS_BGZF_H
 
-void BAMbinSortUnmapped(uint32 iBin, uint nThreads, string dirBAMsort, BGZF *bgzfBAM, Parameters *P);
+void BAMbinSortUnmapped(uint32 iBin, uint nThreads, string dirBAMsort, Parameters &P, Genome &mapGen);
 
 #endif
