@@ -7,19 +7,54 @@
 #include "Genome.h"
 #include "Stats.h"
 #include "OutSJ.h"
-#include <time.h>
 #include "Transcriptome.h"
 #include "BAMoutput.h"
 #include "Quantifications.h"
+#include "ChimericDetection.h"
+
+#include <time.h>
 #include <random>
 
-class ReadAlign : public Genome
+class ReadAlign
 {
     public:
-        Parameters* P; //pointer to the parameters, will be initialized on construction
+         //methods
+        ReadAlign (Parameters& Pin, Genome &genomeIn, Transcriptome *TrIn, int iChunk);//allocate arrays
+        int oneRead();
+        
+        //vars
 
-        //mapping statistics
-        Stats statsRA;
+        Genome &mapGen; //mapped-to-genome structure
+
+        uint iRead;
+        char** Read1;
+        
+        Stats statsRA; //mapping statistics        
+        
+        istream* readInStream[MAX_N_MATES];
+        BAMoutput *outBAMcoord, *outBAMunsorted, *outBAMquant;//sorted by coordinate, unsorted, transcriptomic BAM structure
+        fstream chunkOutChimSAM, *chunkOutChimJunction, chunkOutUnmappedReadsStream[MAX_N_MATES], chunkOutFilterBySJoutFiles[MAX_N_MATES];
+        OutSJ *chunkOutSJ, *chunkOutSJ1;
+
+        ostream* outSAMstream;        
+        uint outBAMbytes; //number of bytes output to SAM/BAM with oneRead
+        char *outBAMarray;//pointer to the (last+1) position of the SAM/BAM output array
+        
+        uint outFilterMismatchNmaxTotal;
+        uint Lread, readLength[MAX_N_MATES], readLengthOriginal[MAX_N_MATES], readLengthPair, readLengthPairOriginal;
+        intScore maxScoreMate[MAX_N_MATES];
+
+        uint readFilesIndex;
+        
+        ReadAlign *waspRA; //ReadAlign for alternative WASP alignment
+        int waspType, waspType1; //alignment ASE-WASP type and
+
+        ReadAlign *peMergeRA; //ReadAlign for merged PE mates
+        
+        ChimericDetection *chimDet;
+        
+    private:
+        Parameters& P; //pointer to the parameters, will be initialized on construction
 
         //quantification
         Transcriptome *chunkTr;
@@ -32,22 +67,13 @@ class ReadAlign : public Genome
         std::uniform_real_distribution<double> rngUniformReal0to1;//initialize in ReadAlign.cpp
 
         //input,output
-        istream* readInStream[MAX_N_MATES];
-        ostream* outSAMstream;
-        OutSJ *chunkOutSJ, *chunkOutSJ1;
-        fstream chunkOutChimSAM, chunkOutChimJunction, chunkOutUnmappedReadsStream[MAX_N_MATES], chunkOutFilterBySJoutFiles[MAX_N_MATES];
-        uint outBAMbytes, outBAMbytes1; //number of bytes output to SAM/BAM with oneRead
-        char *outBAMarray, *outBAMarray1;//pointer to the (last+1) position of the SAM/BAM output array
-        BAMoutput *outBAMcoord, *outBAMunsorted, *outBAMquant;//sorted by coordinate, unsorted, transcriptomic BAM structure
-//        char outBAMoneAlign[MAX_N_MATES][BAMoutput_oneAlignMaxBytes];//tmp array to store BAM alignmnent
-//        uint outBAMoneAlignNbytes[MAX_N_MATES];//number of bytes in the tmp BAM array
+
         char** outBAMoneAlign;
         uint* outBAMoneAlignNbytes;
 
         ostringstream samStreamCIGAR, samStreamSJmotif, samStreamSJintron;
         vector <string> matesCIGAR;
 
-        intScore maxScoreMate[MAX_N_MATES];
         intScore *scoreSeedToSeed, *scoreSeedBest;
         uint *scoreSeedBestInd, *seedChain, *scoreSeedBestMM;
 
@@ -60,13 +86,12 @@ class ReadAlign : public Genome
         Transcript** trArrayPointer; //linear array of transcripts to store all of them from all windows
 
         //read
-        uint iRead, iReadAll, iMate, readFilesIndex;
+        uint iReadAll, iMate;
         char readFilter; //Illumina not passed Y/N
         bool revertStrand; //what to do with the strand, according to strandType and iMate
-        uint Lread, readLength[MAX_N_MATES], readLengthOriginal[MAX_N_MATES], readLengthPair, readLengthPairOriginal;
         uint clip3pNtotal[MAX_N_MATES], clip5pNtotal[MAX_N_MATES], clip3pAdapterN[MAX_N_MATES]; //total number of trimmed bases from 5p,3p
         int readFileType; //file type: 1=fasta; 2=fastq
-        uint outFilterMismatchNmaxTotal;
+
         vector<string>readNameExtra;
 
         char dummyChar[4096];
@@ -74,9 +99,9 @@ class ReadAlign : public Genome
         char** Qual0;
         char** readNameMates;
         char* readName;
-        char** Read1;
         char** Qual1; //modified QSs for scoring
 
+        uint readNmates;
         //split
         uint** splitR;
         uint Nsplit;
@@ -103,21 +128,32 @@ class ReadAlign : public Genome
 
         uint storedLmin, uniqLmax, uniqLmaxInd, multLmax, multLmaxN, multNmin, multNminL, multNmax, multNmaxL;
         uint nTr, nTrMate; // number of transcripts called
-        intScore maxScore, nextWinScore;//maximum alignment score, next best score
-
-        uint chimN, chimRepeat, chimStr, chimMotif;
-        Transcript trChim[MAX_N_CHIMERAS];
-
-        Transcript trA, trA1, *trBest, *trNext, *trInit; //transcript, best tr, next best tr, initialized tr
+        intScore maxScore;//maximum alignment score
+        
+        Transcript trA, trA1, *trBest, *trInit; //transcript, best tr, next best tr, initialized tr
         Transcript ***trAll; //all transcripts for all windows
         uint *nWinTr; //number of recorded transcripts per window
 
+        //old chimeric detection
+        uint chimN, chimRepeat, chimStr;
+        int chimMotif;
+        uint chimRepeat0, chimRepeat1, chimJ0, chimJ1;
+        Transcript trChim[MAX_N_CHIMERAS];
+        //new chimeric detection
+        bool chimRecord; //true if chimeric aligment was detected
+        
         Transcript *alignC, *extendC, *polyAtailC; //alignment rules/conditions
 
         Transcript* trMult[MAX_N_MULTMAP];//multimapping transcripts
         Transcript *alignTrAll;//alignments to transcriptome
-
-        ReadAlign (Parameters* Pin, const Genome &genomeIn, Transcriptome *TrIn, int iChunk);//allocate arrays
+        
+        struct {
+            bool yes;
+            uint nOv;//number of overlapping bases 
+            uint ovS;//first read base of the overlap
+            uint mateStart[2];//mates starts in the merged read
+        } peOv;//PE  mates overlap/merge/remap structure
+        
         void resetN();//resets the counters to 0
         void multMapSelect();
         int mapOneRead();
@@ -131,17 +167,32 @@ class ReadAlign : public Genome
 
         void outputTranscriptSJ(Transcript const &trOut, uint nTrOut, OutSJ *outStream, uint sjReadStartN );
         string outputTranscriptCIGARp(Transcript const &trOut);
-        void outTxtMain(ofstream*,Transcript&);
         int createExtendWindowsWithAlign(uint a1, uint aStr); //extends and windows with one alignment
         void assignAlignToWindow(uint a1, uint aLength, uint aStr, uint aNrep, uint aFrag, uint aRstart,bool aAnchor, uint sjA); //assigns one alignment to a window
-        void stitchPieces(char **R, char **Q, char *G, PackedArray& SA, uint Lread);
-        bool chimericDetection();
+        
+        void mappedFilter();
+        void chimericDetection();
+        bool chimericDetectionOld();
+        void chimericDetectionOldOutput();
+        bool chimericDetectionMult();        
+        void chimericDetectionPEmerged(ReadAlign &seRa);
+//         void chimericDetectionPEmergedTrim();
+                
         void outputAlignments();
-        void stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, char *Q, char *G);//stitches all seeds in one window: iW
         void calcCIGAR(Transcript const &trOut, uint nMates, uint iExMate, uint leftMate);
 
-        int oneRead();
+        void stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R);//stitches all seeds in one window: iW
+        void stitchPieces(char **R, uint Lread);
+
         uint quantTranscriptome (Transcriptome *Tr, uint nAlignG, Transcript **alignG, Transcript *alignT);
+        
+        void copyRead(ReadAlign&);
+        void waspMap();
+        void peOverlapMergeMap();
+        void peMergeMates();
+        void peOverlapSEtoPE(ReadAlign &seRA);
+
+        
 };
 
 #endif
