@@ -12,11 +12,11 @@ void ReadAlign::samAttrNM_MD (Transcript const &trOut, uint iEx1, uint iEx2, uin
     for (uint iex=iEx1;iex<=iEx2;iex++) {
         for (uint ii=0;ii<trOut.exons[iex][EX_L];ii++) {
             char r1=R[ii+trOut.exons[iex][EX_R]];
-            char g1=G[ii+trOut.exons[iex][EX_G]];
+            char g1=mapGen.G[ii+trOut.exons[iex][EX_G]];
             if ( r1!=g1 || r1==4 || g1==4) {
                 ++tagNM;
                 tagMD+=to_string(matchN);
-                tagMD+=P->genomeNumToNT[(uint8) g1];
+                tagMD+=P.genomeNumToNT[(uint8) g1];
                 matchN=0;
             } else {
                 matchN++;
@@ -27,7 +27,7 @@ void ReadAlign::samAttrNM_MD (Transcript const &trOut, uint iEx1, uint iEx2, uin
                 tagNM+=trOut.exons[iex+1][EX_G]-(trOut.exons[iex][EX_G]+trOut.exons[iex][EX_L]);
                 tagMD+=to_string(matchN) + "^";
                 for (uint ii=trOut.exons[iex][EX_G]+trOut.exons[iex][EX_L];ii<trOut.exons[iex+1][EX_G];ii++) {
-                    tagMD+=P->genomeNumToNT[(uint8) G[ii]];
+                    tagMD+=P.genomeNumToNT[(uint8) mapGen.G[ii]];
                 };
                 matchN=0;
             } else if (trOut.canonSJ[iex]==-2) {//insertion
@@ -73,7 +73,7 @@ int bamAttrArrayWrite(string &attr, const char* tagName, char* attrArray ) {
     memcpy(attrArray+3,attr.c_str(),attr.size()+1);//copy string data including \0
     return 3+attr.size()+1;
 };
-int bamAttrArrayWrite(vector<char> &attr, const char* tagName, char* attrArray ) {
+int bamAttrArrayWrite(const vector<char> &attr, const char* tagName, char* attrArray ) {
     attrArray[0]=tagName[0];attrArray[1]=tagName[1];
     attrArray[2]='B';
     attrArray[3]='c';
@@ -81,7 +81,7 @@ int bamAttrArrayWrite(vector<char> &attr, const char* tagName, char* attrArray )
     memcpy(attrArray+4+sizeof(int32),attr.data(),attr.size());//copy array data
     return 4+sizeof(int32)+attr.size();
 };
-int bamAttrArrayWrite(vector<int32> &attr, const char* tagName, char* attrArray ) {
+int bamAttrArrayWrite(const vector<int32> &attr, const char* tagName, char* attrArray ) {
     attrArray[0]=tagName[0];attrArray[1]=tagName[1];
     attrArray[2]='B';
     attrArray[3]='i';
@@ -134,7 +134,7 @@ int bamAttrArrayWriteSAMtags(string &attrStr, char *attrArray) {//write bam reco
 };
 
 template <typename intType>
-int bamAttrArrayWriteInt(intType xIn, const char* tagName, char* attrArray, Parameters *P) {//adapted from samtools
+int bamAttrArrayWriteInt(intType xIn, const char* tagName, char* attrArray, Parameters &P) {//adapted from samtools
     attrArray[0]=tagName[0];attrArray[1]=tagName[1];
     #define ATTR_RECORD_INT(_intChar,_intType,_intValue) attrArray[2] = _intChar; *(_intType*)(attrArray+3) = (_intType) _intValue; return 3+sizeof(_intType)
     int64 x = (int64) xIn;
@@ -149,7 +149,7 @@ int bamAttrArrayWriteInt(intType xIn, const char* tagName, char* attrArray, Para
                 ostringstream errOut;
                 errOut <<"EXITING because of FATAL BUG: integer out of range for BAM conversion: "<< x <<"\n";
                 errOut <<"SOLUTION: contact Alex Dobin at dobin@cshl.edu\n";
-                exitWithError(errOut.str(), std::cerr, P->inOut->logMain, EXIT_CODE_BUG, *P);
+                exitWithError(errOut.str(), std::cerr, P.inOut->logMain, EXIT_CODE_BUG, P);
             };
         };
     } else {
@@ -163,7 +163,7 @@ int bamAttrArrayWriteInt(intType xIn, const char* tagName, char* attrArray, Para
                 ostringstream errOut;
                 errOut <<"EXITING because of FATAL BUG: integer out of range for BAM conversion: "<< x <<"\n";
                 errOut <<"SOLUTION: contact Alex Dobin at dobin@cshl.edu\n";
-                exitWithError(errOut.str(), std::cerr, P->inOut->logMain, EXIT_CODE_BUG, *P);
+                exitWithError(errOut.str(), std::cerr, P.inOut->logMain, EXIT_CODE_BUG, P);
             };
         };
     };
@@ -183,7 +183,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
     //          -13: chimeric alignment, supplemental, soft-clipping
 
 
-    if (P->outSAMmode=="None") return 0; //no SAM/BAM output
+    if (P.outSAMmode=="None") return 0; //no SAM/BAM output
 
     uint32 recSize=0; //record size - total for both mates
     outBAMarrayN[0]=0;
@@ -195,7 +195,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
     uint16 samFLAG=0;
     
 
-    bool flagPaired = P->readNmates==2;
+    bool flagPaired = P.readNmates==2;
 
     uint nMates=1;
     if (alignType<0) {//mapped reads: SAM
@@ -208,17 +208,23 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
     } else {
         nMates=0;
     };
+    
+    uint tLen=0,leftMostMate=0;
+    if (nMates>1 && P.outSAMtlen==2) {
+        tLen=max(trOut.exons[trOut.nExons-1][EX_G]+trOut.exons[trOut.nExons-1][EX_L],trOut.exons[iExMate][EX_G]+trOut.exons[iExMate][EX_L])-min(trOut.exons[0][EX_G],trOut.exons[iExMate+1][EX_G]);
+        leftMostMate=(trOut.exons[0][EX_G]<=trOut.exons[iExMate+1][EX_G] ? 0 : 1);
+    };
 
     uint leftMate=0; //the mate (0 or 1) which is on the left
     if (flagPaired) {
         leftMate=trOut.Str;
     };
                 
-    if (P->outSAMattrPresent.MC) {
+    if (P.outSAMattrPresent.MC) {
         calcCIGAR(trOut, nMates, iExMate, leftMate);
     };
 
-    for (uint imate=0;imate < (alignType<0 ? nMates:P->readNmates);imate++) {
+    for (uint imate=0;imate < (alignType<0 ? nMates:P.readNmates);imate++) {
 
         uint iEx1=0;
         uint iEx2=0;
@@ -234,7 +240,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
         if (alignType>=0) {//this mate is unmapped
             if (mateMapped!=NULL && mateMapped[imate]) continue; //this mate was mapped, do not record it as unmapped
             samFLAG=0x4;
-            if (P->readNmates==2) {//paired read
+            if (P.readNmates==2) {//paired read
                 samFLAG|=0x1 + (imate==0 ? 0x40 : 0x80);
                 if (mateMapped[1-imate]) {//mate mapped
                     if (trOut.Str!=(1-imate))
@@ -242,11 +248,11 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                        samFLAG|=0x20;
                     };
                     mateChr=trOut.Chr;
-                    trChrStart=P->chrStart[mateChr];
+                    trChrStart=mapGen.chrStart[mateChr];
                     mateStart=trOut.exons[0][EX_G] - trChrStart;
                     mateStrand= trOut.Str == (1-imate) ? 0 : 1;
 
-                    if (!trOut.primaryFlag && P->outSAMunmapped.keepPairs)
+                    if (!trOut.primaryFlag && P.outSAMunmapped.keepPairs)
                     {//mapped mate is not primary, keep unmapped mate for each pair, hence need to mark some as not primary
                         samFLAG|=0x100;
                     };
@@ -272,12 +278,14 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
             attrN+=bamAttrArrayWriteInt(trOut.maxScore,"AS",attrOutArray+attrN,P);
             attrN+=bamAttrArrayWriteInt(trOut.nMM,"nM",attrOutArray+attrN,P);
             attrN+=bamAttrArrayWrite((to_string((uint) alignType)).at(0), "uT",attrOutArray+attrN); //cast to uint is only necessary for old compilers
-            if (!P->outSAMattrRG.empty()) attrN+=bamAttrArrayWrite(P->outSAMattrRG.at(readFilesIndex),"RG",attrOutArray+attrN);
+
+            if (!P.outSAMattrRG.empty()) attrN+=bamAttrArrayWrite(P.outSAMattrRG.at(readFilesIndex),"RG",attrOutArray+attrN);
+            
         } else {//this mate is mapped
             if (flagPaired) {//paired reads
                 samFLAG=0x0001;
                 if (iExMate==trOut.nExons-1) {//single mate
-                    if (mateChr>P->nChrReal) samFLAG|=0x0008; //not mapped as pair
+                    if (mateChr>mapGen.nChrReal) samFLAG|=0x0008; //not mapped as pair
                 } else {//properly paired
                     samFLAG|=0x0002; //mapped as pair
                 };
@@ -367,7 +375,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                 packedCIGAR[nCIGAR++]=trimR1<<BAM_CIGAR_OperationShift | (alignType==-12 ? BAM_CIGAR_H : BAM_CIGAR_S);
             };
 
-            MAPQ=P->outSAMmapqUnique;
+            MAPQ=P.outSAMmapqUnique;
             if (nTrOut>=5) {
                 MAPQ=0;
             } else if (nTrOut>=3) {
@@ -387,7 +395,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                         attrN+=bamAttrArrayWriteInt(nTrOut,"NH",attrOutArray+attrN,P);
                         break;
                     case ATTR_HI:
-                        attrN+=bamAttrArrayWriteInt(iTrOut+P->outSAMattrIHstart,"HI",attrOutArray+attrN,P);
+                        attrN+=bamAttrArrayWriteInt(iTrOut+P.outSAMattrIHstart,"HI",attrOutArray+attrN,P);
                         break;
                     case ATTR_AS:
                         attrN+=bamAttrArrayWriteInt(trOut.maxScore,"AS",attrOutArray+attrN,P);
@@ -417,8 +425,41 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                         attrN+=bamAttrArrayWrite(tagMD,"MD",attrOutArray+attrN);
                         break;
                     case ATTR_RG:
-                        attrN+=bamAttrArrayWrite(P->outSAMattrRG.at(readFilesIndex),"RG",attrOutArray+attrN);
+                        attrN+=bamAttrArrayWrite(P.outSAMattrRG.at(readFilesIndex),"RG",attrOutArray+attrN);                    
                         break;
+                    case ATTR_rB:
+                        {
+                            vector <int32> rb;
+                            for (uint ii=iEx1;ii<=iEx2;ii++) {
+                                rb.push_back( (int32) trOut.exons[ii][EX_R]+1 );
+                                rb.push_back( (int32) trOut.exons[ii][EX_R]+trOut.exons[ii][EX_L]);
+                                rb.push_back( (int32) (trOut.exons[ii][EX_G]-mapGen.chrStart[trOut.Chr]+1) );
+                                rb.push_back( (int32) (trOut.exons[ii][EX_G]-mapGen.chrStart[trOut.Chr]+trOut.exons[ii][EX_L]) );                                
+                            };
+                            attrN+=bamAttrArrayWrite(rb,"rB",attrOutArray+attrN);                    
+                        };
+                        break;                             
+                    case ATTR_vG:
+                    {
+                        const vector <int32> &v1=trOut.varGenCoord;
+                        if (v1.size()>0)
+                            attrN+=bamAttrArrayWrite(v1,"vG",attrOutArray+attrN);                                        
+                        break;
+                    };
+                    case ATTR_vA:
+                    {
+                        const vector <char> &v1=trOut.varAllele;
+                        if (v1.size()>0)
+                            attrN+=bamAttrArrayWrite(v1,"vA",attrOutArray+attrN);                                        
+                        break;
+                    };
+                    case ATTR_vW:
+                    {
+                        if (waspType!=-1)
+                            attrN+=bamAttrArrayWrite( (int32) waspType, "vW", attrOutArray+attrN );                                        
+                        break;
+                    };
+                    
                     case ATTR_ch:
                         if (alignType<=-10) 
                         {//chimeric alignment
@@ -433,14 +474,14 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                         break;                        
                     default:
                         ostringstream errOut;
-                        errOut <<"EXITING because of FATAL BUG: unknown/unimplemented SAM atrribute (tag): "<<outSAMattrOrder[ii] <<"\n";
+                        errOut <<"EXITING because of FATAL BUG: unknown/unimplemented SAM/BAM atrribute (tag): "<<outSAMattrOrder[ii] <<"\n";
                         errOut <<"SOLUTION: contact Alex Dobin at dobin@cshl.edu\n";
-                        exitWithError(errOut.str(), std::cerr, P->inOut->logMain, EXIT_CODE_PARAMETER, *P);
+                        exitWithError(errOut.str(), std::cerr, P.inOut->logMain, EXIT_CODE_PARAMETER, P);
                 };
             };
         };
             
-        if (P->readFilesTypeN==10) {
+        if (P.readFilesTypeN==10) {
 //             if (readNameExtra[Mate].size()<1)
 //                 cout << iReadAll <<" " <<readName <<endl; 
             attrN+=bamAttrArrayWriteSAMtags(readNameExtra[Mate],attrOutArray+attrN);
@@ -501,7 +542,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
         };
 
         //4: FLAG<<16|n cigar op; n cigar op is the number of operations in CIGAR.
-        pBAM[4]=( ( ((samFLAG & P->outSAMflagAND) | P->outSAMflagOR) << 16 ) | (nCIGAR) );
+        pBAM[4]=( ( ((samFLAG & P.outSAMflagAND) | P.outSAMflagOR) << 16 ) | (nCIGAR) );
 
         //5: l seq Length of SEQ
         pBAM[5]=seqMateLength;
@@ -509,7 +550,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
         //6: next refID Ref-ID of the next segment (􀀀1  mate refID < n ref)
         if (nMates>1) {
             pBAM[6]=trOut.Chr;
-        } else if (mateChr<P->nChrReal){
+        } else if (mateChr<mapGen.nChrReal){
             pBAM[6]=mateChr;
         } else {
             pBAM[6]=-1;
@@ -518,7 +559,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
         //7: next pos 0-based leftmost pos of the next segment (= PNEXT 􀀀 1)
         if (nMates>1) {
             pBAM[7]=trOut.exons[(imate==0 ? iExMate+1 : 0)][EX_G] - trChrStart;
-        } else if (mateChr<P->nChrReal){
+        } else if (mateChr<mapGen.nChrReal){
             pBAM[7]=mateStart;
         } else {
             pBAM[7]=-1;
@@ -526,9 +567,13 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
 
         //8: tlen Template length (= TLEN)
         if (nMates>1) {
-            int32 tlen=trOut.exons[trOut.nExons-1][EX_G]+trOut.exons[trOut.nExons-1][EX_L]-trOut.exons[0][EX_G];
-            if (imate>0) tlen=-tlen;
-            pBAM[8]=(uint32)tlen;
+            if (P.outSAMtlen==1) {
+                int32 tlen=trOut.exons[trOut.nExons-1][EX_G]+trOut.exons[trOut.nExons-1][EX_L]-trOut.exons[0][EX_G];
+                pBAM[8]=(imate==0 ? tlen : -tlen);
+            } else if (P.outSAMtlen==2) {
+                int32 tlen=(int32)tLen;
+                pBAM[8]=(imate==leftMostMate ? tlen : -tlen);
+            };
         } else {
             pBAM[8]=0;
         };
@@ -548,7 +593,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
         recSize+=(seqMateLength+1)/2;
 
         //Phred base quality (a sequence of 0xFF if absent)
-        if (readFileType==2 && P->outSAMmode != "NoQS") {//output qualtiy
+        if (readFileType==2 && P.outSAMmode != "NoQS") {//output qualtiy
             for (uint32 ii=0; ii<seqMateLength; ii++) {
                 outBAMarray[imate][recSize+ii]=qualOut[ii]-33;
             };
