@@ -8,7 +8,7 @@
 #include "binarySearch2.h"
 #include "ErrorWarning.h"
 
-void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, char *Q, char *G) {//stitches all seeds in one window: iW
+void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R) {//stitches all seeds in one window: iW
 
     for (uint iS1=0;iS1<nWA[iW];iS1++) {
         scoreSeedBest[iS1]=0;
@@ -30,21 +30,21 @@ void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, c
                 score2=\
                     stitchAlignToTranscript(WA[iW][iS2][WA_rStart]+WA[iW][iS2][WA_Length]-1, WA[iW][iS2][WA_gStart]+WA[iW][iS2][WA_Length]-1,\
                                         WA[iW][iS1][WA_rStart], WA[iW][iS1][WA_gStart], WA[iW][iS1][WA_Length], WA[iW][iS1][WA_iFrag],  WA[iW][iS1][WA_sjA], \
-                                        P, R, Q, G, &trA1, outFilterMismatchNmaxTotal);
+                                        P, R, mapGen, &trA1, outFilterMismatchNmaxTotal);
 
-                if (P->outFilterBySJoutStage==2 && trA1.nExons>1)
-                {//junctions have to be present in the filtered set P->sjnovel
+                if (P.outFilterBySJoutStage==2 && trA1.nExons>1)
+                {//junctions have to be present in the filtered set P.sjnovel
                     uint iex=0;
                     if (trA1.canonSJ[iex]>=0 && trA1.sjAnnot[iex]==0)
                     {
                         uint jS=trA1.exons[iex][EX_G]+trA1.exons[iex][EX_L];
                         uint jE=trA1.exons[iex+1][EX_G]-1;
-                        if ( binarySearch2(jS,jE,P->sjNovelStart,P->sjNovelEnd,P->sjNovelN) < 0 ) return;
+                        if ( binarySearch2(jS,jE,P.sjNovelStart,P.sjNovelEnd,P.sjNovelN) < 0 ) return;
                     };
                 };
 
                 //check the length of the iS2 exon. TODO: build the transcripts vs iS1, check the actual exon length
-                bool exonLongEnough = trA1.exons[0][EX_L] >= ( trA1.sjAnnot[0]==0 ? P->alignSJoverhangMin : P->alignSJDBoverhangMin );
+                bool exonLongEnough = trA1.exons[0][EX_L] >= ( trA1.sjAnnot[0]==0 ? P.alignSJoverhangMin : P.alignSJDBoverhangMin );
                 
                 if (exonLongEnough && score2>0 && score2+scoreSeedBest[iS2] > scoreSeedBest[iS1] ) {
                     scoreSeedBest[iS1]=score2+scoreSeedBest[iS2];
@@ -54,12 +54,12 @@ void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, c
             } else {//extend to the left
                 score2=WA[iW][iS1][WA_Length];
                 if ( WA[iW][iS1][WA_rStart]>0 \
-                     && extendAlign(R, Q, G, WA[iW][iS1][WA_rStart]-1, WA[iW][iS1][WA_gStart]-1, -1, -1, WA[iW][iS1][WA_rStart], 100000, 0, outFilterMismatchNmaxTotal, P->outFilterMismatchNoverLmax, \
-                                    P->alignEndsType.ext[WA[iW][iS1][WA_iFrag]][trA.Str], &trA1) ) {//if could extend
+                     && extendAlign(R, mapGen.G, WA[iW][iS1][WA_rStart]-1, WA[iW][iS1][WA_gStart]-1, -1, -1, WA[iW][iS1][WA_rStart], 100000, 0, outFilterMismatchNmaxTotal, P.outFilterMismatchNoverLmax, \
+                                    P.alignEndsType.ext[WA[iW][iS1][WA_iFrag]][trA.Str], &trA1) ) {//if could extend
                     score2 += trA1.maxScore;
                 };
                 
-                bool exonLongEnough = (WA[iW][iS1][WA_Length]+trA1.extendL) >= P->alignSJoverhangMin; //TODO new parameter to control end exons length
+                bool exonLongEnough = (WA[iW][iS1][WA_Length]+trA1.extendL) >= P.alignSJoverhangMin; //TODO new parameter to control end exons length
                 
                 if (exonLongEnough && score2 > scoreSeedBest[iS1] ) {
                     scoreSeedBest[iS1]=score2;
@@ -79,13 +79,13 @@ void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, c
         uint tR2=WA[iW][iS1][WA_rStart]+WA[iW][iS1][WA_Length];
         uint tG2=WA[iW][iS1][WA_gStart]+WA[iW][iS1][WA_Length];
         if ( tR2 < Lread-1 \
-            && extendAlign(R, Q, G, tR2, tG2, +1, +1, Lread-tR2, 100000, scoreSeedBestMM[iS1], outFilterMismatchNmaxTotal, P->outFilterMismatchNoverLmax, \
-                           P->alignEndsType.ext[WA[iW][iS1][WA_iFrag]][1-trA.Str], &trA1) )
+            && extendAlign(R, mapGen.G, tR2, tG2, +1, +1, Lread-tR2, 100000, scoreSeedBestMM[iS1], outFilterMismatchNmaxTotal, P.outFilterMismatchNoverLmax, \
+                           P.alignEndsType.ext[WA[iW][iS1][WA_iFrag]][1-trA.Str], &trA1) )
         {//extend to the right
             scoreSeedBest[iS1]+=trA1.maxScore;
         };
         
-        bool exonLongEnough = (WA[iW][iS1][WA_Length]+trA1.extendL) >= P->alignSJoverhangMin; //TODO new parameter to control end exons length
+        bool exonLongEnough = (WA[iW][iS1][WA_Length]+trA1.extendL) >= P.alignSJoverhangMin; //TODO new parameter to control end exons length
         
         if (exonLongEnough && scoreSeedBest[iS1]>scoreBest) {//record new best transcript
             scoreBest=scoreSeedBest[iS1];
@@ -128,7 +128,7 @@ void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, c
             uint iS1=seedChain[iSc], iS2=seedChain[iSc-1];
             int scoreStitch= stitchAlignToTranscript(WA[iW][iS1][WA_rStart]+WA[iW][iS1][WA_Length]-1, WA[iW][iS1][WA_gStart]+WA[iW][iS1][WA_Length]-1,\
                                         WA[iW][iS2][WA_rStart], WA[iW][iS2][WA_gStart], WA[iW][iS2][WA_Length], WA[iW][iS2][WA_iFrag],  WA[iW][iS2][WA_sjA], \
-                                        P, R, Q, G, &trA, outFilterMismatchNmaxTotal);
+                                        P, R, mapGen, &trA, outFilterMismatchNmaxTotal);
 //            if (scoreStitch>0) {
                 Score+=scoreStitch;
 //           } else {
@@ -142,8 +142,8 @@ void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, c
         {//extend to the left
             trA1=*trInit;
             if ( trA.exons[0][EX_R]>0 \
-                 && extendAlign(R, Q, G, trA.exons[0][EX_R]-1, trA.exons[0][EX_G]-1, -1, -1, trA.exons[0][EX_R], 100000, 0, outFilterMismatchNmaxTotal, P->outFilterMismatchNoverLmax,
-                    P->alignEndsType.ext[trA.exons[0][EX_iFrag]][trA.Str], &trA1) ) {//if could extend
+                 && extendAlign(R, mapGen.G, trA.exons[0][EX_R]-1, trA.exons[0][EX_G]-1, -1, -1, trA.exons[0][EX_R], 100000, 0, outFilterMismatchNmaxTotal, P.outFilterMismatchNoverLmax,
+                    P.alignEndsType.ext[trA.exons[0][EX_iFrag]][trA.Str], &trA1) ) {//if could extend
 
                 trA.add(&trA1);
 
@@ -161,8 +161,8 @@ void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, c
             uint tR2=WA[iW][iS1][WA_rStart]+WA[iW][iS1][WA_Length];
             uint tG2=WA[iW][iS1][WA_gStart]+WA[iW][iS1][WA_Length];
             if ( tR2 < Lread \
-                && extendAlign(R, Q, G, tR2, tG2, +1, +1, Lread-tR2, 100000, scoreSeedBestMM[iS1], outFilterMismatchNmaxTotal, P->outFilterMismatchNoverLmax, \
-                    P->alignEndsType.ext[trA.exons[trA.nExons-1][EX_iFrag]][1-trA.Str], &trA1) ) {//if could extend
+                && extendAlign(R, mapGen.G, tR2, tG2, +1, +1, Lread-tR2, 100000, scoreSeedBestMM[iS1], outFilterMismatchNmaxTotal, P.outFilterMismatchNoverLmax, \
+                    P.alignEndsType.ext[trA.exons[trA.nExons-1][EX_iFrag]][1-trA.Str], &trA1) ) {//if could extend
                     trA.add(&trA1);
                     trA.exons[trA.nExons-1][EX_L] += trA1.extendL;//extend the length of the last exon
             };
@@ -175,7 +175,7 @@ void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, c
 //         uint nMM1=0;
 //         for (uint iex=0;iex<trA.nExons;iex++) {
 //             for (uint ii=0;ii<trA.exons[iex][EX_L];ii++) {
-//                 if ( R[ii+trA.exons[iex][EX_R]]!=G[ii+trA.exons[iex][EX_G]] && G[ii+trA.exons[iex][EX_G]]<4 && R[ii+trA.exons[iex][EX_R]]<4) {
+//                 if ( R[ii+trA.exons[iex][EX_R]]!=G[ii+trA.exons[iex][EX_G]] && mapGen.G[ii+trA.exons[iex][EX_G]]<4 && R[ii+trA.exons[iex][EX_R]]<4) {
 //                     nMM1++;
 //                 };
 //             };
@@ -202,9 +202,9 @@ void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, c
             trA.iFrag=-1;
         };
 
-        if (P->scoreGenomicLengthLog2scale!=0) {//add gap length score
+        if (P.scoreGenomicLengthLog2scale!=0) {//add gap length score
             trA.maxScore += int(ceil( log2( (double) ( trA.exons[trA.nExons-1][EX_G]+trA.exons[trA.nExons-1][EX_L] - trA.exons[0][EX_G]) ) \
-                     * P->scoreGenomicLengthLog2scale - 0.5));
+                     * P.scoreGenomicLengthLog2scale - 0.5));
             trA.maxScore = max(0,trA.maxScore);
         };
 
@@ -226,31 +226,31 @@ void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, c
         else
             trA.sjMotifStrand=0;
 
-        if (trA.intronMotifs[1]>0 && trA.intronMotifs[2]>0 && P->outFilterIntronStrands=="RemoveInconsistentStrands")
+        if (trA.intronMotifs[1]>0 && trA.intronMotifs[2]>0 && P.outFilterIntronStrands=="RemoveInconsistentStrands")
                 return;        
         
-        if (sjN>0 && trA.sjMotifStrand==0 && P->outSAMstrandField=="intronMotif") {//strand not defined for a junction
+        if (sjN>0 && trA.sjMotifStrand==0 && P.outSAMstrandField=="intronMotif") {//strand not defined for a junction
             return;
         };
 
-        if (P->outFilterIntronMotifs=="None") {//no filtering
+        if (P.outFilterIntronMotifs=="None") {//no filtering
 
-        } else if (P->outFilterIntronMotifs=="RemoveNoncanonical") {
+        } else if (P.outFilterIntronMotifs=="RemoveNoncanonical") {
             for (uint iex=0;iex<trA.nExons-1;iex++) {
                 if (trA.canonSJ[iex]==0) return;
             };
-        } else if (P->outFilterIntronMotifs=="RemoveNoncanonicalUnannotated") {
+        } else if (P.outFilterIntronMotifs=="RemoveNoncanonicalUnannotated") {
             for (uint iex=0;iex<trA.nExons-1;iex++) {
                 if (trA.canonSJ[iex]==0 && trA.sjAnnot[iex]==0) return;
             };
         } else {
             ostringstream errOut;
-            errOut << "EXITING because of FATAL INPUT error: unrecognized value of --outFilterIntronMotifs=" <<P->outFilterIntronMotifs <<"\n";
+            errOut << "EXITING because of FATAL INPUT error: unrecognized value of --outFilterIntronMotifs=" <<P.outFilterIntronMotifs <<"\n";
             errOut << "SOLUTION: re-run STAR with --outFilterIntronMotifs = None -OR- RemoveNoncanonical -OR- RemoveNoncanonicalUnannotated\n";
-            exitWithError(errOut.str(),std::cerr, P->inOut->logMain, EXIT_CODE_INPUT_FILES, *P);
+            exitWithError(errOut.str(),std::cerr, P.inOut->logMain, EXIT_CODE_INPUT_FILES, P);
         };
 
-//         if (P->outFilterIntronMotifs=="KeepCanonical" && (trA.intronMotifs[0]>0 || (trA.intronMotifs[1]>0 && trA.intronMotifs[2]>0) ) ) {//keep only conistent canonical introns
+//         if (P.outFilterIntronMotifs=="KeepCanonical" && (trA.intronMotifs[0]>0 || (trA.intronMotifs[1]>0 && trA.intronMotifs[2]>0) ) ) {//keep only conistent canonical introns
 //             return;
 //         };
 
@@ -258,8 +258,8 @@ void ReadAlign::stitchWindowSeeds (uint iW, uint iWrec, bool *WAexcl, char *R, c
         //check exons lenghts including repeats, do not report a transcript with short exons
 //        for (uint isj=0;isj<trA.nExons-1;isj++) {//check exons for min length, if they precede a junction
 //            if ( trA.canonSJ[isj]>=0 &&
-//               ( trA.exons[isj][EX_L] < P->alignSJoverhangMin + trA.shiftSJ[isj][0]
-//              || trA.exons[isj+1][EX_L] < P->alignSJoverhangMin + trA.shiftSJ[isj][1]) ) {
+//               ( trA.exons[isj][EX_L] < P.alignSJoverhangMin + trA.shiftSJ[isj][0]
+//              || trA.exons[isj+1][EX_L] < P.alignSJoverhangMin + trA.shiftSJ[isj][1]) ) {
 //                return;//do not record this transcript in wTr
 //            };
 //        };
