@@ -6,8 +6,8 @@
 #include "binarySearch2.h"
 
 void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, uint tG2, Transcript trA, \
-                        uint Lread, uiWA* WA, char* R, char* Q, char* G, char* sigG,\
-                        Parameters* P, Transcript** wTr, uint* nWinTr, ReadAlign *RA) {
+                        uint Lread, uiWA* WA, char* R, Genome &mapGen, \
+                        Parameters& P, Transcript** wTr, uint* nWinTr, ReadAlign *RA) {
     //recursively stitch aligns for one gene
     //*nWinTr - number of transcripts for the current window
 
@@ -41,9 +41,9 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
             if (trA.rStart>0) {// if transcript does not start at base, extend to the read start
                 trAstep1.reset();
                 uint imate=trA.exons[0][EX_iFrag];
-                if ( extendAlign(R, Q, G, trA.rStart-1, trA.gStart-1, -1, -1, trA.rStart, tR2-trA.rStart+1, \
-                                 trA.nMM, RA->outFilterMismatchNmaxTotal, P->outFilterMismatchNoverLmax, \
-                                 P->alignEndsType.ext[imate][(int)(trA.Str!=imate)], &trAstep1) ) {//if could extend
+                if ( extendAlign(R, mapGen.G, trA.rStart-1, trA.gStart-1, -1, -1, trA.rStart, tR2-trA.rStart+1, \
+                                 trA.nMM, RA->outFilterMismatchNmaxTotal, P.outFilterMismatchNoverLmax, \
+                                 P.alignEndsType.ext[imate][(int)(trA.Str!=imate)], &trAstep1) ) {//if could extend
 
                     trA.add(&trAstep1);
                     Score += trAstep1.maxScore;
@@ -62,9 +62,9 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
             if ( tR2<Lread ) {//extend alignment to the read end
                 trAstep1.reset();
                 uint imate=trA.exons[trA.nExons-1][EX_iFrag];
-                if ( extendAlign(R, Q, G, tR2+1, tG2+1, +1, +1, Lread-tR2-1, tR2-trA.rStart+1, \
-                                 trA.nMM, RA->outFilterMismatchNmaxTotal,  P->outFilterMismatchNoverLmax, \
-                                 P->alignEndsType.ext[imate][(int)(imate==trA.Str)], &trAstep1) ) {//if could extend
+                if ( extendAlign(R, mapGen.G, tR2+1, tG2+1, +1, +1, Lread-tR2-1, tR2-trA.rStart+1, \
+                                 trA.nMM, RA->outFilterMismatchNmaxTotal,  P.outFilterMismatchNoverLmax, \
+                                 P.alignEndsType.ext[imate][(int)(imate==trA.Str)], &trAstep1) ) {//if could extend
 
                     trA.add(&trAstep1);
                     Score += trAstep1.maxScore;
@@ -80,9 +80,9 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
         };
         };
 
-        if (P->alignSoftClipAtReferenceEnds=="No" &&  \
-                ( (trA.exons[trA.nExons-1][EX_G] + Lread-trA.exons[trA.nExons-1][EX_R]) > (P->chrStart[trA.Chr]+P->chrLength[trA.Chr]) || \
-                   trA.exons[0][EX_G]<(P->chrStart[trA.Chr]+trA.exons[0][EX_R]) ) ) {
+        if (P.alignSoftClipAtReferenceEnds=="No" &&  \
+                ( (trA.exons[trA.nExons-1][EX_G] + Lread-trA.exons[trA.nExons-1][EX_R]) > (mapGen.chrStart[trA.Chr]+mapGen.chrLength[trA.Chr]) || \
+                   trA.exons[0][EX_G]<(mapGen.chrStart[trA.Chr]+trA.exons[0][EX_R]) ) ) {
             return; //no soft clipping past the ends of the chromosome
         };
 
@@ -97,15 +97,15 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
         for (uint isj=0;isj<trA.nExons-1;isj++) {//check exons for min length, if they are not annotated and precede a junction
             if ( trA.canonSJ[isj]>=0 ) {//junction
                 if (trA.sjAnnot[isj]==1) {//sjdb
-                    if (  ( trA.exons[isj][EX_L]   < P->alignSJDBoverhangMin && (isj==0            || trA.canonSJ[isj-1]==-3 || (trA.sjAnnot[isj-1]==0 && trA.canonSJ[isj-1]>=0) ) )\
-                       || ( trA.exons[isj+1][EX_L] < P->alignSJDBoverhangMin && (isj==trA.nExons-2 || trA.canonSJ[isj+1]==-3 || (trA.sjAnnot[isj+1]==0 && trA.canonSJ[isj+1]>=0) ) ) )return;
+                    if (  ( trA.exons[isj][EX_L]   < P.alignSJDBoverhangMin && (isj==0            || trA.canonSJ[isj-1]==-3 || (trA.sjAnnot[isj-1]==0 && trA.canonSJ[isj-1]>=0) ) )\
+                       || ( trA.exons[isj+1][EX_L] < P.alignSJDBoverhangMin && (isj==trA.nExons-2 || trA.canonSJ[isj+1]==-3 || (trA.sjAnnot[isj+1]==0 && trA.canonSJ[isj+1]>=0) ) ) )return;
                 } else {//non-sjdb
-                    if (  trA.exons[isj][EX_L] < P->alignSJoverhangMin + trA.shiftSJ[isj][0] \
-                       || trA.exons[isj+1][EX_L] < P->alignSJoverhangMin + trA.shiftSJ[isj][1]   ) return;
+                    if (  trA.exons[isj][EX_L] < P.alignSJoverhangMin + trA.shiftSJ[isj][0] \
+                       || trA.exons[isj+1][EX_L] < P.alignSJoverhangMin + trA.shiftSJ[isj][1]   ) return;
                 };
             };
         };
-        if (trA.nExons>1 && trA.sjAnnot[trA.nExons-2]==1 && trA.exons[trA.nExons-1][EX_L] < P->alignSJDBoverhangMin) return; //this exon was not checkedin the cycle above
+        if (trA.nExons>1 && trA.sjAnnot[trA.nExons-2]==1 && trA.exons[trA.nExons-1][EX_L] < P.alignSJDBoverhangMin) return; //this exon was not checkedin the cycle above
 
         //filter strand consistency
         uint sjN=0;
@@ -125,28 +125,28 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
         else
             trA.sjMotifStrand=0;
 
-        if (trA.intronMotifs[1]>0 && trA.intronMotifs[2]>0 && P->outFilterIntronStrands=="RemoveInconsistentStrands")
+        if (trA.intronMotifs[1]>0 && trA.intronMotifs[2]>0 && P.outFilterIntronStrands=="RemoveInconsistentStrands")
                 return;
 
-        if (sjN>0 && trA.sjMotifStrand==0 && P->outSAMstrandField=="intronMotif") {//strand not defined for a junction
+        if (sjN>0 && trA.sjMotifStrand==0 && P.outSAMstrandField=="intronMotif") {//strand not defined for a junction
             return;
         };
 
-        if (P->outFilterIntronMotifs=="None") {//no filtering
+        if (P.outFilterIntronMotifs=="None") {//no filtering
 
-        } else if (P->outFilterIntronMotifs=="RemoveNoncanonical") {
+        } else if (P.outFilterIntronMotifs=="RemoveNoncanonical") {
             for (uint iex=0;iex<trA.nExons-1;iex++) {
                 if (trA.canonSJ[iex]==0) return;
             };
-        } else if (P->outFilterIntronMotifs=="RemoveNoncanonicalUnannotated") {
+        } else if (P.outFilterIntronMotifs=="RemoveNoncanonicalUnannotated") {
             for (uint iex=0;iex<trA.nExons-1;iex++) {
                 if (trA.canonSJ[iex]==0 && trA.sjAnnot[iex]==0) return;
             };
         } else {
             ostringstream errOut;
-            errOut << "EXITING because of FATAL INPUT error: unrecognized value of --outFilterIntronMotifs=" <<P->outFilterIntronMotifs <<"\n";
+            errOut << "EXITING because of FATAL INPUT error: unrecognized value of --outFilterIntronMotifs=" <<P.outFilterIntronMotifs <<"\n";
             errOut << "SOLUTION: re-run STAR with --outFilterIntronMotifs = None -OR- RemoveNoncanonical -OR- RemoveNoncanonicalUnannotated\n";
-            exitWithError(errOut.str(),std::cerr, P->inOut->logMain, EXIT_CODE_INPUT_FILES, *P);
+            exitWithError(errOut.str(),std::cerr, P.inOut->logMain, EXIT_CODE_INPUT_FILES, P);
         };
 
         {//check mapped length for each mate
@@ -154,7 +154,7 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
             for (uint iex=0;iex<trA.nExons;iex++) {//
                 exl+=trA.exons[iex][EX_L];
                 if (iex==trA.nExons-1 || trA.canonSJ[iex]==-3) {//mate is completed, make the checks
-                    if (nsj>0 && (exl<P->alignSplicedMateMapLmin || exl < (uint) (P->alignSplicedMateMapLminOverLmate*RA->readLength[trA.exons[iex][EX_iFrag]])) ) {
+                    if (nsj>0 && (exl<P.alignSplicedMateMapLmin || exl < (uint) (P.alignSplicedMateMapLminOverLmate*RA->readLength[trA.exons[iex][EX_iFrag]])) ) {
                         return; //do not record this transcript
                     };
                     exl=0;nsj=0;
@@ -164,12 +164,12 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
             };
         };
 
-        if (P->outFilterBySJoutStage==2) {//junctions have to be present in the filtered set P->sjnovel
+        if (P.outFilterBySJoutStage==2) {//junctions have to be present in the filtered set P.sjnovel
             for (uint iex=0;iex<trA.nExons-1;iex++) {
                 if (trA.canonSJ[iex]>=0 && trA.sjAnnot[iex]==0) {
                     uint jS=trA.exons[iex][EX_G]+trA.exons[iex][EX_L];
                     uint jE=trA.exons[iex+1][EX_G]-1;
-                    if ( binarySearch2(jS,jE,P->sjNovelStart,P->sjNovelEnd,P->sjNovelN) < 0 ) return;
+                    if ( binarySearch2(jS,jE,P.sjNovelStart,P.sjNovelEnd,P.sjNovelN) < 0 ) return;
                 };
             };
         };
@@ -187,9 +187,9 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
             if ( trA.exons[iexM2-1][EX_G] + trA.exons[iexM2-1][EX_L] > trA.exons[iexM2][EX_G] ) {//mates overlap - check consistency of junctions
 
                 if (trA.exons[0][EX_G] > \
-                    trA.exons[iexM2][EX_G]+trA.exons[0][EX_R]+P->alignEndsProtrude.nBasesMax) return; //LeftMateStart > RightMateStart + allowance
+                    trA.exons[iexM2][EX_G]+trA.exons[0][EX_R]+P.alignEndsProtrude.nBasesMax) return; //LeftMateStart > RightMateStart + allowance
                 if (trA.exons[iexM2-1][EX_G]+trA.exons[iexM2-1][EX_L] > \
-                   trA.exons[trA.nExons-1][EX_G]+Lread-trA.exons[trA.nExons-1][EX_R]+P->alignEndsProtrude.nBasesMax) return; //LeftMateEnd   > RightMateEnd +allowance            
+                   trA.exons[trA.nExons-1][EX_G]+Lread-trA.exons[trA.nExons-1][EX_R]+P.alignEndsProtrude.nBasesMax) return; //LeftMateEnd   > RightMateEnd +allowance            
                 
                 //check for junctions consistency
                 uint iex1=1, iex2=iexM2+1; //last exons of the junction
@@ -216,13 +216,14 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
             };//mates overlap - check consistency of junctions
         };//check for correct overlap between mates
 
-        if (P->scoreGenomicLengthLog2scale!=0) {//add gap length score
+        if (P.scoreGenomicLengthLog2scale!=0) {//add gap length score
             Score += int(ceil( log2( (double) ( trA.exons[trA.nExons-1][EX_G]+trA.exons[trA.nExons-1][EX_L] - trA.exons[0][EX_G]) ) \
-                     * P->scoreGenomicLengthLog2scale - 0.5));
+                     * P.scoreGenomicLengthLog2scale - 0.5));
             Score = max(0,Score);
         };
 
         //calculate some final values for the transcript
+
         trA.roStart = (trA.roStr == 0) ? trA.rStart : Lread - trA.rStart - trA.rLength;
         trA.maxScore=Score;
 
@@ -233,14 +234,20 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
             trA.iFrag=-1;
         };
 
-        if (       Score+P->outFilterMultimapScoreRange >= wTr[0]->maxScore \
-                || ( trA.iFrag>=0 && Score+P->outFilterMultimapScoreRange >= RA->maxScoreMate[trA.iFrag] ) \
-                || P->chimSegmentMin>0) {
+        //Variation
+        Score+=trA.variationAdjust(mapGen, R);
+        
+        trA.maxScore=Score;
+        
+        // transcript has been finalized, compare the score and record
+        if (       Score+P.outFilterMultimapScoreRange >= wTr[0]->maxScore \
+                || ( trA.iFrag>=0 && Score+P.outFilterMultimapScoreRange >= RA->maxScoreMate[trA.iFrag] ) \
+                || P.pCh.segmentMin>0) {
                 //only record the transcripts within the window that are in the Score range
                 //OR within the score range of each mate
                 //OR all transcript if chimeric detection is activated
 
-//             if (P->alignEndsType.in=="EndToEnd") {//check that the alignment is end-to-end
+//             if (P.alignEndsType.in=="EndToEnd") {//check that the alignment is end-to-end
 //                 uint rTotal=trA.rLength+trA.lIns;
 // //                 for (uint iex=1;iex<trA.nExons;iex++) {//find the inside exons
 // //                     rTotal+=trA.exons[iex][EX_R]-trA.exons[iex-1][EX_R];
@@ -286,7 +293,7 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
                 };
                 wTr[iTr]=pTr; //the new transcript pointer is now at *nWinTr+1, move it into the iTr
                 *(wTr[iTr])=trA;
-                if (*nWinTr<P->alignTranscriptsPerWindowNmax) {
+                if (*nWinTr<P.alignTranscriptsPerWindowNmax) {
                     (*nWinTr)++; //increment number of transcripts per window;
                 } else {
                         //"WARNING: too many recorded transcripts per window: iRead="<<RA->iRead<< "\n";
@@ -303,7 +310,7 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
     Transcript trAi=trA; //trA copy with this align included, to be used in the 1st recursive call of StitchAlign
     if (trA.nExons>0) {//stitch, a transcript has already been originated
 
-        dScore=stitchAlignToTranscript(tR2, tG2, WA[iA][WA_rStart], WA[iA][WA_gStart], WA[iA][WA_Length], WA[iA][WA_iFrag],  WA[iA][WA_sjA], P, R, Q, G, &trAi, RA->outFilterMismatchNmaxTotal);
+        dScore=stitchAlignToTranscript(tR2, tG2, WA[iA][WA_rStart], WA[iA][WA_gStart], WA[iA][WA_Length], WA[iA][WA_iFrag],  WA[iA][WA_sjA], P, R, mapGen, &trAi, RA->outFilterMismatchNmaxTotal);
         //TODO check if the new stitching creates too many MM, quit this transcript if so
 
     } else { //this is the first align in the transcript
@@ -315,7 +322,7 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
 
             trAi.nExons=1; //recorded first exon
 
-            for (uint ii=0;ii<WA[iA][WA_Length];ii++) dScore+=int(Q [ WA[iA][WA_rStart] + ii ]); //sum all the scores
+            for (uint ii=0;ii<WA[iA][WA_Length];ii++) dScore+=scoreMatch; //sum all the scores
 
             trAi.nMatch=WA[iA][WA_Length]; //# of matches
 
@@ -330,7 +337,7 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
         if ( WA[iA][WA_Nrep]==1 ) trAi.nUnique++; //unique piece
         if ( WA[iA][WA_Anchor]>0 ) trAi.nAnchor++; //anchor piece piece
 
-        stitchWindowAligns(iA+1, nA, Score+dScore, WAincl, WA[iA][WA_rStart]+WA[iA][WA_Length]-1, WA[iA][WA_gStart]+WA[iA][WA_Length]-1, trAi, Lread, WA, R, Q, G, sigG, P, wTr, nWinTr, RA);
+        stitchWindowAligns(iA+1, nA, Score+dScore, WAincl, WA[iA][WA_rStart]+WA[iA][WA_Length]-1, WA[iA][WA_gStart]+WA[iA][WA_Length]-1, trAi, Lread, WA, R, mapGen, P, wTr, nWinTr, RA);
     } else {
 
     };
@@ -338,7 +345,7 @@ void stitchWindowAligns(uint iA, uint nA, int Score, bool WAincl[], uint tR2, ui
     //also run a transcript w/o including this align
     if (WA[iA][WA_Anchor]!=2 || trA.nAnchor>0) {//only allow exclusion if this is not the last anchor, or other anchors have been used
         WAincl[iA]=false;
-        stitchWindowAligns(iA+1, nA, Score, WAincl, tR2, tG2, trA, Lread, WA, R, Q, G, sigG, P, wTr, nWinTr, RA);
+        stitchWindowAligns(iA+1, nA, Score, WAincl, tR2, tG2, trA, Lread, WA, R, mapGen, P, wTr, nWinTr, RA);
     };
     return;
 };
